"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EvmPlatform = void 0;
const connect_sdk_1 = require("@wormhole-foundation/connect-sdk");
const ethers_1 = require("ethers");
const chain_1 = require("./chain");
const platformUtils_1 = require("./platformUtils");
// forces EvmPlatform to implement Platform
var _ = EvmPlatform;
/**
 * @category EVM
 */
// Provides runtime concrete value
var EvmPlatform;
(function (EvmPlatform) {
    EvmPlatform.platform = 'Evm';
    EvmPlatform.network = connect_sdk_1.DEFAULT_NETWORK;
    EvmPlatform.conf = (0, connect_sdk_1.networkPlatformConfigs)(EvmPlatform.network, EvmPlatform.platform);
    EvmPlatform.nativeTokenId = platformUtils_1.EvmUtils.nativeTokenId, EvmPlatform.isNativeTokenId = platformUtils_1.EvmUtils.isNativeTokenId, EvmPlatform.isSupportedChain = platformUtils_1.EvmUtils.isSupportedChain, EvmPlatform.getDecimals = platformUtils_1.EvmUtils.getDecimals, EvmPlatform.getBalance = platformUtils_1.EvmUtils.getBalance, EvmPlatform.getBalances = platformUtils_1.EvmUtils.getBalances, EvmPlatform.sendWait = platformUtils_1.EvmUtils.sendWait, EvmPlatform.getCurrentBlock = platformUtils_1.EvmUtils.getCurrentBlock, EvmPlatform.chainFromChainId = platformUtils_1.EvmUtils.chainFromChainId, EvmPlatform.chainFromRpc = platformUtils_1.EvmUtils.chainFromRpc, EvmPlatform.getTokenImplementation = platformUtils_1.EvmUtils.getTokenImplementation;
    function setConfig(_network, _conf) {
        EvmPlatform.conf = _conf ? _conf : (0, connect_sdk_1.networkPlatformConfigs)(EvmPlatform.network, EvmPlatform.platform);
        EvmPlatform.network = _network;
        return EvmPlatform;
    }
    EvmPlatform.setConfig = setConfig;
    function getRpc(chain) {
        if (chain in EvmPlatform.conf)
            return ethers_1.ethers.getDefaultProvider(EvmPlatform.conf[chain].rpc);
        throw new Error('No configuration available for chain: ' + chain);
    }
    EvmPlatform.getRpc = getRpc;
    function getChain(chain) {
        if (chain in EvmPlatform.conf)
            return new chain_1.EvmChain(EvmPlatform.conf[chain]);
        throw new Error('No configuration available for chain: ' + chain);
    }
    EvmPlatform.getChain = getChain;
    async function getProtocol(protocol) {
        try {
            switch (protocol) {
                case 'TokenBridge':
                case 'AutomaticTokenBridge':
                    const tb = await (0, connect_sdk_1.loadProtocolModule)(EvmPlatform.platform, 'tokenbridge');
                    if (EvmPlatform.platform + protocol in tb)
                        return tb[EvmPlatform.platform + protocol];
                case 'CircleBridge':
                case 'AutomaticCircleBridge':
                    const cb = await (0, connect_sdk_1.loadProtocolModule)(EvmPlatform.platform, 'cctp');
                    if (EvmPlatform.platform + protocol in cb)
                        return cb[EvmPlatform.platform + protocol];
                case 'WormholeCore':
                    const core = await (0, connect_sdk_1.loadProtocolModule)(EvmPlatform.platform, 'core');
                    if (EvmPlatform.platform + protocol in core)
                        return core[EvmPlatform.platform + protocol];
                default:
                    throw new Error('Protocol not supported: ' + protocol);
            }
        }
        catch (e) {
            console.error('Error loading ' + protocol, e);
            throw e;
        }
    }
    EvmPlatform.getProtocol = getProtocol;
    async function getWormholeCore(rpc) {
        return (await getProtocol('WormholeCore')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getWormholeCore = getWormholeCore;
    async function getTokenBridge(rpc) {
        return (await getProtocol('TokenBridge')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getTokenBridge = getTokenBridge;
    async function getAutomaticTokenBridge(rpc) {
        return (await getProtocol('TokenBridge')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getAutomaticTokenBridge = getAutomaticTokenBridge;
    async function getCircleBridge(rpc) {
        return (await getProtocol('CircleBridge')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getCircleBridge = getCircleBridge;
    async function getAutomaticCircleBridge(rpc) {
        return (await getProtocol('AutomaticCircleBridge')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getAutomaticCircleBridge = getAutomaticCircleBridge;
    async function parseTransaction(chain, rpc, txid) {
        const wc = await getWormholeCore(rpc);
        return wc.parseTransaction(txid);
    }
    EvmPlatform.parseTransaction = parseTransaction;
})(EvmPlatform || (exports.EvmPlatform = EvmPlatform = {}));
//# sourceMappingURL=platform.js.map