import { networkPlatformConfigs, DEFAULT_NETWORK, loadProtocolModule, } from '@wormhole-foundation/connect-sdk';
import { ethers } from 'ethers';
import { EvmChain } from './chain';
import { EvmUtils } from './platformUtils';
// forces EvmPlatform to implement Platform
var _ = EvmPlatform;
/**
 * @category EVM
 */
// Provides runtime concrete value
export var EvmPlatform;
(function (EvmPlatform) {
    EvmPlatform.platform = 'Evm';
    EvmPlatform.network = DEFAULT_NETWORK;
    EvmPlatform.conf = networkPlatformConfigs(EvmPlatform.network, EvmPlatform.platform);
    EvmPlatform.nativeTokenId = EvmUtils.nativeTokenId, EvmPlatform.isNativeTokenId = EvmUtils.isNativeTokenId, EvmPlatform.isSupportedChain = EvmUtils.isSupportedChain, EvmPlatform.getDecimals = EvmUtils.getDecimals, EvmPlatform.getBalance = EvmUtils.getBalance, EvmPlatform.getBalances = EvmUtils.getBalances, EvmPlatform.sendWait = EvmUtils.sendWait, EvmPlatform.getCurrentBlock = EvmUtils.getCurrentBlock, EvmPlatform.chainFromChainId = EvmUtils.chainFromChainId, EvmPlatform.chainFromRpc = EvmUtils.chainFromRpc, EvmPlatform.getTokenImplementation = EvmUtils.getTokenImplementation;
    function setConfig(_network, _conf) {
        EvmPlatform.conf = _conf ? _conf : networkPlatformConfigs(EvmPlatform.network, EvmPlatform.platform);
        EvmPlatform.network = _network;
        return EvmPlatform;
    }
    EvmPlatform.setConfig = setConfig;
    function getRpc(chain) {
        if (chain in EvmPlatform.conf)
            return ethers.getDefaultProvider(EvmPlatform.conf[chain].rpc);
        throw new Error('No configuration available for chain: ' + chain);
    }
    EvmPlatform.getRpc = getRpc;
    function getChain(chain) {
        if (chain in EvmPlatform.conf)
            return new EvmChain(EvmPlatform.conf[chain]);
        throw new Error('No configuration available for chain: ' + chain);
    }
    EvmPlatform.getChain = getChain;
    async function getProtocol(protocol) {
        try {
            switch (protocol) {
                case 'TokenBridge':
                case 'AutomaticTokenBridge':
                    const tb = await loadProtocolModule(EvmPlatform.platform, 'tokenbridge');
                    if (EvmPlatform.platform + protocol in tb)
                        return tb[EvmPlatform.platform + protocol];
                case 'CircleBridge':
                case 'AutomaticCircleBridge':
                    const cb = await loadProtocolModule(EvmPlatform.platform, 'cctp');
                    if (EvmPlatform.platform + protocol in cb)
                        return cb[EvmPlatform.platform + protocol];
                case 'WormholeCore':
                    const core = await loadProtocolModule(EvmPlatform.platform, 'core');
                    if (EvmPlatform.platform + protocol in core)
                        return core[EvmPlatform.platform + protocol];
                default:
                    throw new Error('Protocol not supported: ' + protocol);
            }
        }
        catch (e) {
            console.error('Error loading ' + protocol, e);
            throw e;
        }
    }
    EvmPlatform.getProtocol = getProtocol;
    async function getWormholeCore(rpc) {
        return (await getProtocol('WormholeCore')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getWormholeCore = getWormholeCore;
    async function getTokenBridge(rpc) {
        return (await getProtocol('TokenBridge')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getTokenBridge = getTokenBridge;
    async function getAutomaticTokenBridge(rpc) {
        return (await getProtocol('TokenBridge')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getAutomaticTokenBridge = getAutomaticTokenBridge;
    async function getCircleBridge(rpc) {
        return (await getProtocol('CircleBridge')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getCircleBridge = getCircleBridge;
    async function getAutomaticCircleBridge(rpc) {
        return (await getProtocol('AutomaticCircleBridge')).fromProvider(rpc, EvmPlatform.conf);
    }
    EvmPlatform.getAutomaticCircleBridge = getAutomaticCircleBridge;
    async function parseTransaction(chain, rpc, txid) {
        const wc = await getWormholeCore(rpc);
        return wc.parseTransaction(txid);
    }
    EvmPlatform.parseTransaction = parseTransaction;
})(EvmPlatform || (EvmPlatform = {}));
//# sourceMappingURL=platform.js.map